function [IC, FC, TrajH, TrajT] = GE_Zeni(Data, headerTraj,FootMrk, side, fs, mrkDyn, TurnsNotDec, MrkGaps)

%% GE detection based on Zeni et al. (2008)
% Zeni Jr, J. A., Richards, J. G., & Higginson, J. S. (2008).
% Two simple methods for determining gait events during treadmill and
% overground walking using kinematic data.
% Gait & posture, 27(4), 710-714.
% https://doi.org/10.1016/j.gaitpost.2007.07.007
%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

%% Mrks on the pelvic segment are filled
if isempty(mrkDyn)
    % PlugInGait is used
    markers = fieldnames(Data.((headerTraj)));
    mrkDyn = [markers(contains(markers, 'ASIS'));markers(contains(markers, 'PSIS'))];
    checkPelvis = false;
else
    checkPelvis = true;
end
if checkPelvis
    if isempty(TurnsNotDec) && length(MrkGaps) == 1 && ~contains(MrkGaps,'REF') && ~contains(MrkGaps,'WRIST') && contains(MrkGaps,'DYN')
        check = false;
    else
        check = true;
    end
else
    if isempty(TurnsNotDec) && isempty(find(contains(MrkGaps,'ASIS'), 1)) && isempty(find(contains(MrkGaps,'PSIS'), 1)) 
        check = false;
    else
        check = true;
    end
end
for k = 1:length(mrkDyn)
    Data.(headerTraj).(mrkDyn{k}) = fillgaps(Data.(headerTraj).(mrkDyn{k}));
end

%% Define a LRF using the skin-markers placed on the MIMU located on the Lower Back
if check
    if checkPelvis
        for i = 1:length(Data.(headerTraj).DYNA0)
            zML(i,:) = (Data.(headerTraj).DYNAX(i,:) - Data.(headerTraj).DYNA0(i,:))/...
                norm(Data.(headerTraj).DYNAX(i,:) - Data.(headerTraj).DYNA0(i,:));          % Medio-lateral axis
            v(i,:) = (Data.(headerTraj).DYNAY(i,:) - Data.(headerTraj).DYNA0(i,:))/...
                norm(Data.(headerTraj).DYNAY(i,:) - Data.(headerTraj).DYNA0(i,:));
            xAP(i,:) = cross(v(i,:),zML(i,:))/norm(cross(v(i,:),zML(i,:)));                 % Anterior-Posterior axis
            
            yV(i,:) = cross(zML(i,:), xAP(i,:))/norm(cross(zML(i,:), xAP(i,:)));            % Vertical axis
            
            R(:,:,i) = [xAP(i,:)' yV(i,:)' zML(i,:)'];                                      % LRF Rotation Matix
            
            CosTheta(i) = dot(yV(i,:),[0 0 1])/(norm(yV(i,:))*norm([0 0 1]));               % Define inclination of the LRF with respect to the vertical (GRF)
            ThetaInDegrees(i) = acosd(CosTheta(i));
            
            if ~isnan(ThetaInDegrees(i))
                Rm = rotz(ThetaInDegrees(i));
            else
                %% if there are gaps, previous information is used!!!
                posNow = find(~isnan(ThetaInDegrees)); posNow = posNow(end);
                Rm = rotz(ThetaInDegrees(posNow));
                %%
            end
            R_align(:,:,i) = R(:,:,i)*Rm;                                                   % Alignment of the LRF with respect to the vertical direction
        end
        t = Data.(headerTraj).DYNA0;
        % Origin of the LRF
    
    else
        %% Define a LRF using the skin-markers placed on the pelvis
        % Pelvic Anatomical Reference Frame (ARF), as defined in Cappozzo et al., 1995
        midASIS = (Data.(headerTraj).RASIS+Data.(headerTraj).LASIS)/2; % midpoint between the anterior superior iliac spines
        midPSIS = (Data.(headerTraj).RPSIS+Data.(headerTraj).LPSIS)/2; % midpoint between the posterior superior iliac spines
        
        for i = 1:length(Data.(headerTraj).LASIS)
            zML(i,:) = (Data.(headerTraj).RASIS(i,:) - Data.(headerTraj).LASIS(i,:))/...
                norm(Data.(headerTraj).RASIS(i,:) - Data.(headerTraj).LASIS(i,:));          % Medio-lateral axis
            v(i,:) = (midASIS(i,:) - midPSIS(i,:))/...
                norm(midASIS(i,:) - midPSIS(i,:));
            yV(i,:) = cross(zML(i,:), v(i,:))/norm(cross(zML(i,:), v(i,:)));                % Vertical axis
            xAP(i,:) = cross(yV(i,:),zML(i,:))/norm(cross(yV(i,:),zML(i,:)));               % Anterior-Posterior axis
            
            R(:,:,i) = [xAP(i,:)' yV(i,:)' zML(i,:)'];                                      % LRF Rotation Matix
            
            CosTheta(i) = dot(yV(i,:),[0 0 1])/(norm(yV(i,:))*norm([0 0 1]));               % Define inclination of the LRF with respect to the vertical (GRF)
            ThetaInDegrees(i) = acosd(CosTheta(i));
            Rm = rotz(ThetaInDegrees(i));
            R_align(:,:,i) = R(:,:,i)*Rm;                                                   % Alignment of the LRF with respect to the vertical direction
        end
        t = midPSIS;                    % Origin of the LRF        
    end % check which markers are placed on the pelvis
    
else
    % there are GAPS on the pelvic segment
    if checkPelvis
        if strcmp(MrkGaps, 'DYNAY')
            for i = 1:length(Data.(headerTraj).DYNA0)
                zML(i,:) = (Data.(headerTraj).DYNAX(i,:) - Data.(headerTraj).DYNA0(i,:))/...
                    norm(Data.(headerTraj).DYNAX(i,:) - Data.(headerTraj).DYNA0(i,:));          % Medio-lateral axis
                v(i,:) = (Data.(headerTraj).DYNAREF(i,:) - Data.(headerTraj).DYNA0(i,:))/...
                    norm(Data.(headerTraj).DYNAREF(i,:) - Data.(headerTraj).DYNA0(i,:));
                xAP(i,:) = cross(v(i,:),zML(i,:))/norm(cross(v(i,:),zML(i,:)));                 % Anterior-Posterior axis
                
                yV(i,:) = cross(zML(i,:), xAP(i,:))/norm(cross(zML(i,:), xAP(i,:)));            % Vertical axis
                
                R(:,:,i) = [xAP(i,:)' yV(i,:)' zML(i,:)'];                                      % LRF Rotation Matix
                
                CosTheta(i) = dot(yV(i,:),[0 0 1])/(norm(yV(i,:))*norm([0 0 1]));               % Define inclination of the LRF with respect to the vertical (GRF)
                ThetaInDegrees(i) = acosd(CosTheta(i));
                Rm = rotz(ThetaInDegrees(i));
                R_align(:,:,i) = R(:,:,i)*Rm;                                                   % Alignment of the LRF with respect to the vertical direction
            end
            t = Data.(headerTraj).DYNA0;                                                        % Origin of the LRF
            
        elseif strcmp(MrkGaps, 'DYNA0')
            warning('This condition has not been properly tested yet')
            %         for i = 1:length(Data.(headerTraj).DYNAREF)
            %             zML(i,:) = (Data.(headerTraj).DYNAX(i,:) - Data.(headerTraj).DYNAREF(i,:))/...
            %                 norm(Data.(headerTraj).DYNAX(i,:) - Data.(headerTraj).DYNAREF(i,:));          % Medio-lateral axis
            %             v(i,:) = (Data.(headerTraj).DYNAY(i,:) - Data.(headerTraj).DYNA0(i,:))/...
            %                 norm(Data.(headerTraj).DYNAY(i,:) - Data.(headerTraj).DYNA0(i,:));
            %             xAP(i,:) = cross(v(i,:),zML(i,:))/norm(cross(v(i,:),zML(i,:)));                 % Anterior-Posterior axis
            %
            %             yV(i,:) = cross(zML(i,:), xAP(i,:))/norm(cross(zML(i,:), xAP(i,:)));            % Vertical axis
            %
            %             R(:,:,i) = [xAP(i,:)' yV(i,:)' zML(i,:)'];                                      % LRF Rotation Matix
            %
            %             CosTheta(i) = dot(yV(i,:),[0 0 1])/(norm(yV(i,:))*norm([0 0 1]));               % Define inclination of the LRF with respect to the vertical (GRF)
            %             ThetaInDegrees(i) = acosd(CosTheta(i));
            %             Rm = rotz(ThetaInDegrees(i));
            %             R_align(:,:,i) = R(:,:,i)*Rm;                                                   % Alignment of the LRF with respect to the vertical direction
            %         end
            %         t = Data.(headerTraj).DYNA0;                                                        % Origin of the LRF
        else % MrkGaps == DYNAX
            warning('This condition has not been properly tested yet')
        end
    else
        %% Define a LRF using the skin-markers placed on the pelvis
        % Pelvic Anatomical Reference Frame (ARF), as defined in Cappozzo et al., 1995
        midASIS = (Data.(headerTraj).RASIS+Data.(headerTraj).LASIS)/2; % midpoint between the anterior superior iliac spines
        midPSIS = (Data.(headerTraj).RPSIS+Data.(headerTraj).LPSIS)/2; % midpoint between the posterior superior iliac spines
        
        for i = 1:length(Data.(headerTraj).LASIS)
            zML(i,:) = (Data.(headerTraj).RASIS(i,:) - Data.(headerTraj).LASIS(i,:))/...
                norm(Data.(headerTraj).RASIS(i,:) - Data.(headerTraj).LASIS(i,:));          % Medio-lateral axis
            v(i,:) = (midASIS(i,:) - midPSIS(i,:))/...
                norm(midASIS(i,:) - midPSIS(i,:));
            yV(i,:) = cross(zML(i,:), v(i,:))/norm(cross(zML(i,:), v(i,:)));                % Vertical axis
            xAP(i,:) = cross(yV(i,:),zML(i,:))/norm(cross(yV(i,:),zML(i,:)));               % Anterior-Posterior axis
            
            R(:,:,i) = [xAP(i,:)' yV(i,:)' zML(i,:)'];                                      % LRF Rotation Matix
            
            CosTheta(i) = dot(yV(i,:),[0 0 1])/(norm(yV(i,:))*norm([0 0 1]));               % Define inclination of the LRF with respect to the vertical (GRF)
            ThetaInDegrees(i) = acosd(CosTheta(i));
            Rm = rotz(ThetaInDegrees(i));
            R_align(:,:,i) = R(:,:,i)*Rm;                                                   % Alignment of the LRF with respect to the vertical direction
        end
        t = midPSIS;                    % Origin of the LRF 
    end
end

%% TOE and HEEL markers represented in the LRF aligned with the vertical
for i = 1:length(zML)
    HEEL_l(i,:) = (R_align(:,:,i)'*(Data.(headerTraj).(strcat(side,FootMrk{1}))(i,:) - t(i,:))')';   % from GRF to LRF
    TOE_l(i,:) = (R_align(:,:,i)'*(Data.(headerTraj).(strcat(side,FootMrk{2}))(i,:) - t(i,:))')';    % from GRF to LRF
end
MrkL.HEEL = HEEL_l;
MrkL.TOE = TOE_l;
MrkL.COM = t;
MrkL.R = R_align;

TrajH = HEEL_l(:,1);
TrajT = TOE_l(:,1);

%% check Local mrk traj velocity
vTrajH = diff(TrajH)*fs;
vTrajT = diff(TrajT)*fs;

IC = [];
FC = [];

%% IC identification
[~, HS_n] = findpeaks(TrajH,'MinPeakProminence',0.1);
v_Heel = diff(TrajH)/(1/fs);

if ~isempty(HS_n)
    for i = 1:length(HS_n)
        if (v_Heel(HS_n(i))<0) && (v_Heel(HS_n(i)-1)>0)
            IC = [IC; HS_n(i)];
        end
    end
else
    fprintf('IC events not found. Consider using another method!\n');
end

%% TO identification
ws1 = 0.08*fs;
[~,FC_n] = findpeaks(-TrajT,'MinPeakDistance',ws1, 'MinPeakProminence',0.2);
v_Toe = diff(TrajT)/(1/fs);

if ~isempty(FC_n)
    for i = 1:length(FC_n)
        if (v_Toe(FC_n(i))>0) && (v_Toe(FC_n(i)-1)<0)
            FC = [FC; FC_n(i)];
        end
    end
else
    fprintf('FC events not found with Zeni algo. Consider using another method!\n');
end
end